"""
functions for setting up seckerwiki
"""
import os
import getpass

EXAMPLE_CONTENTS = """---
encrypted-journal-path: Personal/Personal-Management/Journal/Encrypted
receipts-path: Personal/Personal-Management/Receipts
"""

AUTH_CONTENTS = """---
password: {0}
"""


def setup():
  """
  Create config file and auth file
  """
  cfg_path =  os.path.join(os.getcwd(), "wiki.yml")
  auth_path = os.path.expanduser("~/.config/seckerwiki/credentials")

  if os.path.exists(cfg_path):
    print(f"Error: seckerwiki config file already exists at {cfg_path}")
    return False

  if os.path.exists(auth_path):
    print(f"Error: seckerwiki auth file already exists at {auth_path}")
    return False
    
  with open(cfg_path, 'w') as f:
      f.write(EXAMPLE_CONTENTS)
  os.chmod(cfg_path,0o600)

  print(f"Configuration file written to {cfg_path}")

  journal_password = getpass.getpass(prompt="Enter journal passphrase: ")


  # create auth directory
  os.makedirs(os.path.dirname(auth_path), exist_ok=True)
    
  with open(auth_path, 'w') as f:
      f.write(AUTH_CONTENTS.format(journal_password))
  os.chmod(auth_path,0o600)

  print(f"Credentials file written to {auth_path}")


import os
import yaml

# Terminal Colours
# https://stackoverflow.com/questions/287871/how-to-print-colored-text-in-terminal-in-python
class bcolors:
  HEADER = '\033[95m'
  OKBLUE = '\033[94m'
  OKGREEN = '\033[92m'
  WARNING = '\033[93m'
  FAIL = '\033[91m'
  ENDC = '\033[0m'
  BOLD = '\033[1m'
  UNDERLINE = '\033[4m'

class NoPasswordError(Exception):
  pass

def get_journal_key():
  """
  :return the symmetric key stored in ~/.config/seckerwiki/credentials
  """
  path = os.path.expanduser("~/.config/seckerwiki/credentials")

  if not os.path.exists(path):
    raise FileNotFoundError(f"Couldn't find credentials file at {path}")
  
  with open(os.path.abspath(path), 'r') as f:
    auth = yaml.safe_load(f)

    if not 'password' in auth or auth['password'] is None or auth['password'] == "":
      raise NoPasswordError(f"Password isn't defined in credentials file! ({path})")
    return auth['password']

def file_contains_markers(path, start, end):
  """
  :return true if both :start and :end are in the file 
  """

  with open(path, 'r') as f_in:
    contents = f_in.read()
    return start in contents and end in contents

def replace_between_markers(file, start, end, content):
  # TODO check if content is the same - if it is, skip
  with open(file, 'r') as f_in:
    contents = f_in.readlines()

  start_line = contents.index(start)+1
  end_line = contents.index(end)

  del contents[start_line:end_line]

  contents.insert(start_line, content)
  
  with open(file, 'w') as f_out:
    f_out.writelines(contents)

  


#!/usr/bin/env python3
import argparse
import os
import sys
import yaml

from seckerwiki.commands.lecture import lecture
from seckerwiki.commands.git import commit, log, sync
from seckerwiki.commands.receipt import receipt
from seckerwiki.commands.journal import journal
from seckerwiki.commands.setup import setup
from seckerwiki.commands.toc import toc
from seckerwiki.commands.stats import stats

def main():

  parser = argparse.ArgumentParser()

  # Global arguments
  parser.add_argument("-c", "--config", help="custom path to wiki config file", default=None)

  # Add all the subparsers 
  subparsers = parser.add_subparsers()

  # Disabled because I don't need lecture functionality anymore
  # lecture_parser = subparsers.add_parser('lecture', help='create new lecture slides')
  # lecture_parser.add_argument('-b', '--blank', action='store_true', help='Create blank lecture slides.')
  # lecture_parser.set_defaults(func=lecture)

  setup_parser = subparsers.add_parser('setup', help='setup wiki CLI')
  setup_parser.set_defaults(func=setup)

  log_parser = subparsers.add_parser('log', help='show git log')
  log_parser.set_defaults(func=log)

  commit_parser = subparsers.add_parser('commit', help='commit wiki')
  commit_parser.add_argument('-y', action='store_true', help='Don\'t ask for confirmation before committing')
  commit_parser.set_defaults(func=commit)

  sync_parser = subparsers.add_parser('sync', help='sync with remote repo')
  sync_parser.set_defaults(func=sync)

  journal_parser = subparsers.add_parser('journal', help='make journal entry')
  journal_parser.add_argument('-e', '--encrypt', action='store_true', help='encrypt all unencrypted journal entries')
  journal_parser.add_argument('-d', '--decrypt', help='decrypt filename')
  journal_parser.set_defaults(func=journal)

  toc_parser = subparsers.add_parser('toc', help='generate table of contents in files')
  toc_parser.set_defaults(func=toc)

  stats_parser = subparsers.add_parser('stats', help="wiki stats")
  stats_parser.set_defaults(func=stats)

  args = parser.parse_args()

  # print help and exit if no arguments supplied
  if not hasattr(args, 'func'):
    parser.print_help()
    exit(0)

  # run setup script without config
  if args.func is setup:
    setup() 
    sys.exit()

  # Load custom config if defined in env var
  cfg = None
  try:
    cfg_file = os.path.expanduser(args.config) if args.config else os.path.join(os.getcwd(), "wiki.yml")
    with open(os.path.abspath(cfg_file), 'r') as f:
      cfg = yaml.safe_load(f)
  except FileNotFoundError:
    print(f"Config file not found at {cfg_file}. Have you ran `wiki setup`?")
    sys.exit(1)

  # Change directory if custom dir provided
  if args.config:
    path = os.path.abspath(os.path.dirname(os.path.expanduser(args.config)))
    print(f"Running command in {path}")
    os.chdir(path)

  # Run the subcommand
  args.func(cfg, args)


if __name__ == "__main__":
  main()


import os
import subprocess
from PyInquirer import prompt

# Commit command
def commit(cfg, args):
  """
  Commit the files
  args:
  -y : don't confirm
  """

  def convert(line):
    """
    Convert line into a tuple of (root folder, filename)
    """
    line = str(line.decode('utf-8'))
    path = line.split(os.sep)
    root = path[0] if len(path) > 0 else ""
    filename = path[-1]
    return (root, filename)

  # run git diff command
  # if changed more than 3 files, list folders instead
  output = subprocess.run(['git', 'diff', '--name-only', 'HEAD'], stdout=subprocess.PIPE).stdout.splitlines()
  changed = list(map(convert, output))

  if not changed:
    print("No files detected")
    return

  # Make a message using the filenames. If its too long,
  # Set the commit title to the root folders, then the commit description to each of the files changed
  message = " ".join([pair[1] for pair in changed])
  if len(message) > 50:
    message = " ".join(set([pair[0] for pair in changed])) + "\n\n" + "\n".join([pair[1] for pair in changed])

  print("Committing with messsage: \n{0}".format(message))

  confirm = [
    {
      'name': 'confirm',
      'message': 'commit?',
      'type': 'confirm',
      'default': True
    }
  ]

  # Prompt for confirmation
  if not args.y and not prompt(confirm)['confirm']:
    return

  # make the commit
  os.system("git commit -am \"{0}\"".format(message))


def sync(cfg, args):
  """
  git pull and git push
  """
  print("Syncing...")

  # Forcefully encrypt the journal
  # print("Encrypting Journal...")
  # encrypt_journal(None)

  os.system("git pull -r && git push")


# Log command
def log(cfg, args):
  os.system(
    "git log --graph --pretty=\"%Cred%h%Creset -%C(yellow)%d%Creset %s %Cgreen(%cr) %C(bold blue)<%an>%Creset\" --abbrev-commit")



import os
from datetime import date
import sys
import getpass

from seckerwiki.util import bcolors, get_journal_key

def journal(cfg, args):
  if args.encrypt:
    encrypt_journal(cfg, args)
  elif args.decrypt:
    decrypt_journal(cfg, args)
  else:
    generate_note(cfg, args)


def generate_note(cfg, args):
  """
  Generate an empty note
  """
  today = date.today().isoformat()
  filename = '{0}.md'.format(today)
  text = '# Encrypted Note -  {0}\n\n - '.format(today)

  path = os.path.join(cfg['encrypted-journal-path'], filename)

  with open(path, 'a') as f:
    f.write(text)
    print("Generated Journal Entry: ", path)

def encrypt_journal(cfg, args):
  journal_dir = os.path.join(os.getcwd(), cfg['encrypted-journal-path'])

  # check if journal path exists
  if not os.path.isdir(journal_dir):
    print("Journal directory not found: {0}".format(journal_dir))
    return

  print("Encrypting Journal Entries...")
  for root, dirs, files in os.walk(journal_dir):
    for file in files:
      # only encrypt markdown files
      if not file.endswith(".md"):
        print("Skipping: {0}{1}{2}".format(bcolors.OKBLUE, file, bcolors.ENDC))
        continue

      print("Encrypting: {0}{1}{2}".format(bcolors.OKGREEN, file, bcolors.ENDC))
      os.system(
        "gpg -c --armor --batch --passphrase {0} {1}".format(get_journal_key(), os.path.join(root, file)))
      # delete the markdown file
      os.remove(os.path.join(root, file))


def decrypt_journal(cfg, args):
  path = os.path.join(os.getcwd(), cfg['encrypted-journal-path'], args.decrypt)

  # check if entry path exists
  if not os.path.isfile(path):
    print("Journal entry not found: {0}".format(path))
    return

  print("Decrypting Journal Entry: {0}{1}{2}".format(bcolors.OKGREEN, path, bcolors.ENDC), file=sys.stderr)
  return_value = os.system("gpg -d --armor --batch --passphrase {0} {1}".format(get_journal_key(), path))

  if (return_value >> 8) != 0:
    # decryption failed, ask for password
    password = getpass.getpass(prompt="Decryption failed. Manually enter password: ")

    os.system("gpg -d --armor --batch --passphrase {0} {1}".format(password, path))


import os
import subprocess
from seckerwiki.util import bcolors

def stats(cfg, args):
  """
  Print some useless/useful information about the wiki, for showing off purposes
  """

  commit_count = int(subprocess.check_output(['git', 'rev-list', '--all', '--count']))

  loc_map = {}
  
  for root, dirs, files in os.walk(os.getcwd()):
    for _file in [f for f in files if f.endswith(".md")]:
      path = os.path.join(root, _file)
      with open(path, 'r') as f_in:
        loc_map[path] = sum(1 for line in f_in if line.strip())

  total_lines = sum(loc_map.values())
  largest_files = "\n".join(f"    {f}" for f in sorted(loc_map, key=loc_map.get, reverse=True)[:3])

  output = f"""
{bcolors.BOLD}Wiki Stats:{bcolors.ENDC}
  {bcolors.OKGREEN}Commits made: {bcolors.ENDC} {commit_count}
  {bcolors.OKGREEN}Total Lines: {bcolors.ENDC} {total_lines:,}
  {bcolors.OKGREEN}Largest files: {bcolors.ENDC} 
{largest_files}
  """

  print(output)


#!/usr/bin/env python3

"""
This script is pretty outdated, and won't be updated anymore because I don't have any need to take uni notes.
It won't work after the refactoring work started in e8d1664bfb4c0ef

It expects a certain wiki structure, and something like the following in the wiki config file:

courses:
  tri-1:
  - COMP424
  - NWEN438
  - ENGR401
  tri-2:
  - NWEN439
  - SWEN430
  - ENGR441
  full-year:
  - ENGR489

If anyone else wanted to use this I'd recommend taking out all the stuff that requires following my wiki structure and make it a bit more customizable.

"""

import argparse
import tempfile
import re
import requests
import os
from datetime import date

import pdf2image
import yaml


def convert_to_pdf(input_path):
  """
  Use external tools to convert the powerpoint file to a pdf.

  Returns:
    path of converted file
  """
  path, extension = os.path.splitext(input_path)

  if extension not in ['.ppt', '.pptx']:
    raise ValueError("{0} not a valid powerpoint extension".format(extension))

  renamed_path = path + ".pdf"

  # unoconv doesn't support writing to a file, so pipe to a file
  print("Powerpoint file detected, converting...")
  os.system("unoconv -f pdf --stdout \"{0}\" > \"{1}\"".format(input_path, renamed_path))
  print("converted from {0} to {1}".format(input_path, renamed_path))

  return renamed_path


def download_file(url):
  """
  Download and return temporary file
  """
  print("url detected, downloading...")

  response = requests.get(url)

  # detect file type from MIME-TYPE of request
  content_type = response.headers['content-type']
  if content_type == 'application/pdf':
    file_type = ".pdf"
  elif content_type == "application/vnd.ms-powerpoint":
    file_type = ".ppt"
  elif content_type == "application/vnd.openxmlformats-officedocument.presentationml.presentation":
    file_type = ".pptx"
  else:
    print("couldn't figure out type of downloaded file. aborting")
    raise

  print("downloaded {0}".format(file_type))

  # write to temporary file
  temp = tempfile.NamedTemporaryFile(suffix=file_type)
  temp.write(response.content)

  return temp


def is_url(_str):
  """return true if the str input is a URL."""
  ur = re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\), ]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', _str)
  return len(ur) > 0


def create_filename(lecture_num, title):
  """
  Create a filename from a title string.

  Converts spaces to dashes and makes everything lowercase

  Returns:
   lecture filename
  """
  return "lecture-{:02d}{}.md".format(int(lecture_num), "" if not title else '-' + title.lower().replace(" ", "-"))


def create_markdown_header(lecture_num, title, url):
  now = date.today().isoformat()
  url_link = "[Online lecture slides link]({0})".format(url) if url else ""
  return "# Lecture {0}: {1}\n({2})\n\n{3}\n\n".format(lecture_num, title, now, url_link)


def get_tri_from_course(course, courses):
  """
  Given a courses dict, return the trimester of the :param course
  """
  for tri in courses:
    for _course in courses[tri]:
      if course == _course:
        return tri


def get_wiki_folder(config, lecture_num, course):
  """
  Return tuple of (lecture file location, markdown images path)
  """

  tri = get_tri_from_course(course, config['courses'])
  lecture_path = os.path.join(config['wiki-root'], "Uni", tri.capitalize(), course, "Lectures")
  images_path = os.path.join(lecture_path, "images", "Lecture-{:02d}".format(int(lecture_num)))

  return [lecture_path, images_path]


def generate_lecture(cfg, pdf: str, course: str, lecture_num: int, title:str=None, blank=False):
  """
  Generate markdown based lecture slides from a pdf.

  Args:
    cfg: config file
    pdf (str): URL of the PDF or pathname
    course (str): course code
    lecture_num (int): lecture number
    title (str): lecture title
    blank (bool): whether or not to fill in a blank lecture
  """
  # combine the possible course selections from both tris
  courses = [item for key in cfg['courses'].keys() for item in cfg['courses'][key]]

  # Cache the downloaded file so it doesn't get deleted by GC
  downloaded = None

  # Download the file if script was provided a URL
  if is_url(pdf):
    downloaded = download_file(pdf)
    file = downloaded.name
    url = pdf
  else:
    file = pdf
    url = None

  # convert to pdf if it isn't one already
  extension = os.path.splitext(file)[1]
  pdf = convert_to_pdf(file) if extension in ['.ppt', '.pptx'] else file

  # convert the file to a series of images
  print("converting PDF to images...")
  with tempfile.TemporaryDirectory() as image_path:
    images = pdf2image.convert_from_path(
      pdf_path=pdf,
      dpi=60,
      output_folder=image_path,
      fmt="png",
      thread_count=4
    )
  print("done.")

  # Get paths
  lecture_path, images_path = get_wiki_folder(cfg, lecture_num, course)

  # create dirs if they don't exist
  if not os.path.exists(images_path):
    os.makedirs(images_path, exist_ok=True)

  # Create top of lecture page
  lecture_markdown = create_markdown_header(lecture_num, title, url)

  # Add images to subfolder and lecture page
  for image_num, image in enumerate(images):
    # get filenames and paths
    image_name = "lecture-{0}-{1}.png".format(lecture_num, image_num)
    image_path = os.path.join(images_path, image_name)

    # save to file
    image.save(image_path)

    # save to markdown
    lecture_markdown += "![image]({0})\n".format("images/Lecture-{0:02d}/{1}".format(int(lecture_num), image_name))

    # add note
    lecture_markdown += "### Slide {0} notes \n\n".format(image_num)

  # Finally, write out to the file
  filename = os.path.join(lecture_path, create_filename(lecture_num, title))
  with open(filename, 'a') as f:
    f.write(lecture_markdown)

  print("Generated Lecture in {0}".format(filename))


if __name__ == '__main__':
  # load config file
  try:
    cfg_file = os.environ['WIKI_CONFIG'] if 'WIKI_CONFIG' in os.environ else os.path.expanduser('~/.personal.yml')
    with open(cfg_file, 'r') as ymlfile:
      cfg = yaml.safe_load(ymlfile)
  except FileNotFoundError:
    print("Config file not found. Have you ran `wiki setup`?")

  courses = [item for key in cfg['courses'].keys() for item in cfg['courses'][key]]

  # parse arguments
  parser = argparse.ArgumentParser()
  parser.add_argument("pdf", help="input pdf file or url")
  parser.add_argument("course", help="course", choices=courses)
  parser.add_argument("lecture_num", help="lecture number")
  parser.add_argument("--title", help="Lecture title")
  parser.add_argument("--blank", help="blank lecture")

  args = parser.parse_args()

  generate_lecture(cfg, args.pdf, args.course, args.lecture_num, args.title, args.blank)



import os

from PyInquirer import prompt

def receipt(cfg, args):
  options = [
    {
      'type': 'input',
      'message': 'enter product name',
      'name': 'name'
    },
    {
      'type': 'input',
      'message': 'enter date bought',
      'name': 'bought'
    }
  ]

  answers = prompt(options)

  # TODO make a popup here choosing the receipt fil


"""
This is pretty outdated and is only kept for reference - see scripts/lecture_gen.py 
"""
import glob
import os
import re

from PyInquirer import prompt

from seckerwiki.scripts.lecture_gen import generate_lecture


def lecture(cfg, args):
    # merge all courses from every category into a single list
    courses = [item for key in cfg['courses'].keys() for item in cfg['courses'][key]]

    questions = [
        {
            'type': 'input',
            'name': 'input_file',
            'message': 'Enter path of lecture pdf, or url'
        },
        {
            'type': 'list',
            'name': 'course',
            'message': 'Enter course code',
            'choices': courses
        },
        {
            'type': 'input',
            'name': 'lecture_num',
            'message': 'Enter lecture number',
            'validate': lambda x: x.isdigit(),
            'default': lambda answers: get_latest_lecture_num(cfg, answers['course'])
        },
        {
            'type': 'input',
            'name': 'title',
            'message': 'Enter lecture title',
        }
    ]
    answers = prompt(questions)
    generate_lecture(cfg, answers['input_file'], answers['course'], answers['lecture_num'], title=answers['title'])


def get_latest_lecture_num(cfg, course):
    """
    Return the latest lecture number from the filenames in course
    """
    tri = get_tri_from_course(course, cfg['courses'])
    lecture_dir = os.path.join(cfg['wiki-root'], "Uni", tri, course, "Lectures")

    print(lecture_dir)
    files = [f for f in glob.glob(lecture_dir + "**/*.md", recursive=False)]

    # if no files exist, return 0
    if len(files) < 1:
        print("No recent lectures found.")
        return "1"

    last_file = sorted(files)[-1]

    print("Most recent lecture: " + last_file)

    # use regex to match number (get last as most likely to be lecture num if alphabetised)
    last = re.findall(r'\d+', last_file)[-1]

    # suggest the latest lecture num + 1
    suggested = int(last) + 1 if last.isdigit() else "none"

    return str(suggested)


def get_tri_from_course(course, courses):
    """
    Given a courses dict, return the trimester of the :param course
    """
    for tri in courses:
        for _course in courses[tri]:
            if course == _course:
                return tri.capitalize()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['seckerwiki', 'seckerwiki.commands', 'seckerwiki.scripts']

package_data = \
{'': ['*']}

install_requires = \
['PyInquirer>=1.0.3,<2.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'pdf2image>=1.16.0,<2.0.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['wiki = seckerwiki.wiki:main']}

setup_kwargs = {
    'name': 'seckerwiki',
    'version': '2.0.1',
    'description': 'A collection of scripts used to manage my personal Foam workspace',
    'long_description': None,
    'author': 'Benjamin Secker',
    'author_email': 'benjamin.secker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bsecker/wiki/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import os
from seckerwiki.util import file_contains_markers, replace_between_markers


"""
Table of contents generator
"""

TOC_START = "<!--BEGIN_TOC-->\n"
TOC_END = "<!--END_TOC-->\n"

def toc(cfg, args):
  """
  Generates a table of contents in every file that has the TOC_START and TOC_END markers.
  """
  
  excluded_dirs = cfg['toc-excluded-dirs']
  count = 0

  for root, dirs, files in os.walk(os.getcwd()):
    # filter out excluded dirs https://stackoverflow.com/questions/19859840/excluding-directories-in-os-walk
    dirs[:] = [d for d in dirs if d not in excluded_dirs]

    # filter to markdown files
    md_files = [file for file in files if file.endswith(".md")]

    for _file in md_files:
      file_path = os.path.join(root, _file)
      if file_contains_markers(file_path, TOC_START, TOC_END):
        toc = generate_toc(root, dirs, files)

        replace_between_markers(file_path, TOC_START, TOC_END, toc)

        count += 1

  print(f"Updated {count} files. Make sure to re-run `foam janitor` to update link compatibility.")

def generate_toc(root, subdirs, files):
  # Generate directory links. My standard is to have a "contents-page" with the same name (lowercased) of the directory it's contained in

  subdir_lines = []
  files_lines = []

  # Generate subdir links
  for subdir in subdirs:
    filename = subdir.lower() + ".md"
    if os.path.isfile(os.path.join(root, subdir, filename)):

      subdir_path = f"./{subdir}/{filename}"
      subdir_lines.append(f"[{subdir}]({subdir_path})")

  # generate file links
  for _file in files:
    if _file.endswith(".md"):
      files_lines.append(f"[{_file.replace('.md', '')}](./{_file})")

  return render_contents(subdir_lines, files_lines)

def render_contents(subdirs, pages):

  subdir_lines="\n".join([f"- {subdir}" for subdir in subdirs])
  pages_lines="\n".join([f"- {page}" for page in pages])

  toc = ""

  if subdirs:
    toc += "Subdirectories:\n"
    toc += subdir_lines + "\n\n"

  if pages:
    toc += "Pages:\n"
    toc += pages_lines + "\n"

  # render page
  return toc